/*************************************************************************

	FILE:       BOX.CPP

	AUTHOR:     Brian Tegart, C.E.T.

	DATE:       April 30, 1995

	DESCRIPTION:    Code for Graphics rectangle

	COPYRIGHT (c) 1995 Tegart Software

==========================================================================

	Revision:   0.0

	Date:       April 30, 1995

	By:     Brian Tegart, C.E.T.

	Action:     Creation and modification from old file

**************************************************************************/

/************/
/* Includes */
/************/
#include <alloc.h>
#include "box.h"
#include "line.h"
#include "mouse.h"
#include "screen.h"

/***********/
/* Defines */
/***********/

/********************/
/* Global variables */
/********************/

/**********************/
/* External variables */
/**********************/
extern Mouse mouse;

/******************************/
/* Static function prototypes */
/******************************/

/********************/
/* Static functions */
/********************/

/********************/
/* Public functions */
/********************/

/*------------------------------------------------------------------------

Prototype:  Box::Box(int InitX1, int InitY1, int InitX2, int InitY2,
			int InitColor, int InitTopColor,
			int InitBottomColor, int InitSaveBGState,
			BYTE *InitScreen,
			int InitNumEdgeLines = DEFAULT_LINES)

Description:    Box constructor

Input:      InitX1      - initial X1 coordinate
		InitY1      - initial Y1 coordinate
		InitX2      - initial X2 coordinate
		InitY2      - initial Y2 coordinate
		InitColor   - initial face color
		InitTopColor    - initial top edge color
		InitBottomColor - initial bottom edge color
		InitSaveBGState - flag indicating wether to save
					background when showing
		InitScreen  - pointer to screen buffer for box
		InitNumEdgeLines- number of edge lines for box

Modified:   none

Output:     none

Returns:    none

Side effects:   none

-------------------------------------------------------------------------*/
Box::Box(int InitX1, int InitY1, int InitX2, int InitY2,
		int InitColor, int InitTopColor,
		int InitBottomColor, int InitSaveBGState,
		BYTE *InitScreen,
		int InitNumEdgeLines)
	: Rectangle(InitX1, InitY1, InitX2, InitY2, InitColor)
{
	screen = InitScreen;
	SaveBG = InitSaveBGState;
	SavedBG = FALSE;
	background = NULL;
	TopColor = InitTopColor;
	BottomColor = InitBottomColor;
	NumEdgeLines = InitNumEdgeLines;
} // Box Constructor

/*------------------------------------------------------------------------

Prototype:  void Box::SaveBackground()

Description:    Save area of screen where box coordiates are

Input:      none

Modified:   none

Output:     none

Returns:    none

Side effects:   none

-------------------------------------------------------------------------*/
void Box::SaveBackground()
{
	if(SavedBG)
		delete background;
	background = new BYTE[(X2-X1+1) * (Y2-Y1+1)];
	mouse.MaybeHide();
	getbox(background,X1,Y1,X2-X1+1,Y2-Y1+1,screen);
	mouse.MaybeShow();
	SavedBG = TRUE;
} // Box::SaveBackground

/*------------------------------------------------------------------------

Prototype:  void Box::RestoreBackground()

Description:    Restore saved area of screen where box coordiates are

Input:      none

Modified:   none

Output:     none

Returns:    none

Side effects:   none

-------------------------------------------------------------------------*/
void Box::RestoreBackground()
{
	if(!SavedBG)
		return;
	mouse.MaybeHide();
	putbox(background,X1,Y1,X2-X1+1,Y2-Y1+1,screen);
	mouse.MaybeShow();
	delete background;
	SavedBG = FALSE;
} // Box::RestoreBackground

/*------------------------------------------------------------------------

Prototype:	void Box::DrawEdges()

Description:    Draw the edges around a box

Input:     	none

Modified:   	none

Output:    	none

Returns:   	none

Side effects:   none

-------------------------------------------------------------------------*/
void Box::DrawEdges()
{
	int i;

	mouse.MaybeHide();
	for(i=0;i<NumEdgeLines;i++)
	{
		vlinedraw(X1+i, Y1, Y2-i, TopColor, screen);
		hlinedraw(X1, Y1+i, X2-i, TopColor, screen);
		vlinedraw(X2-i, Y1+i, Y2, BottomColor, screen);
		hlinedraw(X1+i, Y2-i, X2, BottomColor, screen);
	}
	mouse.MaybeShow();
} // Box::DrawEdges

/*------------------------------------------------------------------------

Prototype:	Box::SwapEdgeColors()

Description:	Swaps edge colors

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:	none

-------------------------------------------------------------------------*/
void Box::SwapEdgeColors()
{
	int TempColor = TopColor;

	TopColor = BottomColor;
	BottomColor = TempColor;
	if(Visible)
		ReShow();
} // Box::SwapEdgeColors

/*------------------------------------------------------------------------

Prototype:	void Box::Show()

Description:    Show box, saving background if necessary

Input:		none

Modified:	none

Output:     	none

Returns:	none

Side effects:   none

-------------------------------------------------------------------------*/
void Box::Show()
{
	if(Visible)
		return;
	if(SavedBG)
		delete background;
	if(SaveBG)
		SaveBackground();
	Visible = TRUE;
	ReShow();
} // Box::Show

/*------------------------------------------------------------------------

Prototype:	void Box::ReShow()

Description:    ReShows box (refresh, such as when edge colors change)

Input:		none

Modified:	none

Output:     	none

Returns:	none

Side effects:   none

-------------------------------------------------------------------------*/
void Box::ReShow()
{
	if(!Visible)
		return;
	mouse.MaybeHide();
	boxdraw(X1, Y1, X2, Y2, Color, screen);
	DrawEdges();
	mouse.MaybeShow();
} // Box::ReShow

/*------------------------------------------------------------------------

Prototype:	void Box::Hide()

Description:    Hide box, restoring background if necessary

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:   none

-------------------------------------------------------------------------*/
void Box::Hide()
{
	if(!Visible)
		return;
	mouse.MaybeHide();
	if(SavedBG)
		RestoreBackground();
	else
		boxdraw(X1, Y1, X2, Y2, 0, screen);
	mouse.MaybeShow();
	Visible = FALSE;
} // Box::Hide

/*------------------------------------------------------------------------

Prototype:	void Box::MoveTo()

Description:    Move box, hiding and re-showing if necessary

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:   none

-------------------------------------------------------------------------*/
void Box::MoveTo(int NewX1, int NewY1, int NewX2, int NewY2)
{
	int WasVis = FALSE;

	if(Visible)
	{
		Hide();
		WasVis = TRUE;
	}
	Rectangle::MoveTo(NewX1, NewY1, NewX2, NewY2);
	if(WasVis == TRUE)
		Show();
} // Box::MoveTo

/*------------------------------------------------------------------------

Prototype:	void Box::~Box()

Description:    Box destructor

Input:		none

Modified:	none

Output:		none

Returns:	none

Side effects:   none

-------------------------------------------------------------------------*/
Box::~Box()
{
	Hide();
} // Box::~Box
