#include <alloc.h>
#include <dos.h>
#include <fcntl.h>
#include <io.h>
#include <mem.h>
#include <stdio.h>
#include "pcx.h"
#include "rtrncode.h"
#include "screen.h"

#define IMAGE_SIZE 64000	// PCX image size

// Pcx constructor
Pcx::Pcx(BYTE *InitScreen)
{
	screen = InitScreen;
	Pcx_Struct = new struct pcx_struct;
	image_loaded = 0;
	Visible = 0;
} // Pcx constructor

// Pcx::load_image decompresses bitmap and stores in buffer
int Pcx::load_image(int pcxfile)
{
	const int BYTEMODE = 0, RUNMODE = 1;	// Encoding modes
	const int BUFLEN = 5 * 1024;		// Buffer for data read from disk
	int mode = BYTEMODE;	// Current encoding mode being used.
	int readlen;		// Number of bytes read from file
	static BYTE outbyte;		// Next byte for buffer
	static BYTE bytecount;	// Counter for runs
	static BYTE buffer[BUFLEN];	// Disk read buffer

	// Allocate mem for bitmap buffer and return -1 if an error occurs
	if((Pcx_Struct->image = new BYTE[IMAGE_SIZE]) == NULL)
		return (NO_MEM);
	image_loaded = 1;
	int bufptr = 0;		// Point to start of buffer
	readlen = 0;			// Zero bytes read so far

	// Create pointer to start of image
	BYTE *image_ptr = Pcx_Struct->image;

	// Loop for entire length of bitmap buffer
	for(long i=0;i<IMAGE_SIZE;i++)
	{
		if(mode==BYTEMODE)
		{
			if(bufptr>=readlen)		// If past end of buffer...
			{
				bufptr = 0;			//  	point to start
				// Read more bytes from file into buffer, if no more bytes
				// left, break out of loop
				if((readlen=read(pcxfile, buffer, BUFLEN))==0)
					break;
			}
			outbyte=buffer[bufptr++];		// Next byte of bitmap
			if(outbyte > 0xbf)		// run length flag (192 or greater)
			{
				// Calculate the number of bytes in run
				bytecount = (int)((int)outbyte & 0x3f);  // Same as - 192
				if(bufptr >= readlen)		// If past end of buffer,
				{
					bufptr = 0;			//	point to start
					// Read more bytes from file into buffer;
					//	if no more bytes left, break out of loop
					if((readlen=read(pcxfile, buffer, BUFLEN))==0)
						break;
				}
				outbyte = buffer[bufptr++];	// Next byte of bitmap
				// Switch to run-length mode
				if(--bytecount > 0)
					mode = RUNMODE;
			}
		}
		// Switch to individual byte mode
		else if(--bytecount == 0)
			mode = BYTEMODE;
		// Add next byte to bitmap buffer
		*image_ptr++ = outbyte;
	}
	return GOOD;
} // Pcx::load_image

// Pcx:load_palette loads color reg values from file into palette array
void Pcx::load_palette(int pcxfile)
{
	// Seek to start of palette, which is always 768 bytes from end of file
	lseek(pcxfile, -768L, SEEK_END);
	// Read all 768 bytes of palette into array
	read(pcxfile, Pcx_Struct->palette, 3*256);
	// Adjust for required bit shifts
	for(int i=0;i<256;i++)
		for(int j=0;j<3;j++)
			Pcx_Struct->palette[i*3+j] >>= 2;
} // Pcx::load_palette

// Pcx::load loads PCX data from file FILENAME into structure PCX
int Pcx::load(char far *filename)
{
	int result;

	if((infile = open(filename, O_BINARY)) == NULL)
		return (NO_FILE_OPEN);

	// Move file pointer to start of header
	lseek(infile, 0L, SEEK_SET);

	// Read header
	read(infile, &(Pcx_Struct->header), sizeof(pcx_header));

	// Decompress bitmap and place in buffer, returning nonzero value
	//	if error occurs
	result = load_image(infile);
	if(result!=GOOD)
	{
		close(infile);
		return (result);
	}

	// Decompress palette and store in array
	load_palette(infile);

	close(infile);
	return GOOD;
} // Pcx::load

// Function to compress bitmap stored in PCX->IMAGE into compressed
//	image buffer PCX->CIMAGE
void Pcx::compress()
{
	int runlength;
	WORD ptr=0;				// Pointer to position in pcx->image
	WORD cptr=0;			// Pointer to position in pcx->cimage
	BYTE value;
	BYTE *cbuffer;

	cbuffer = Pcx_Struct->cimage;

	// Begin compression
	while(ptr < IMAGE_SIZE)
	{
		// get next byte from image
		value = Pcx_Struct->image[ptr];
		runlength = 0;
		while((value==Pcx_Struct->image[ptr]) && (runlength < 63))
		{
			runlength++;
			ptr++;
		}
		if((value>=192)&&(runlength==1))
			runlength = 193;
		if(runlength==1)
			runlength = 0;
		if(runlength)
		{
			if(runlength!=193)
				runlength += 192;
			cbuffer[cptr] = (BYTE)runlength;
			cptr++;
		}
		cbuffer[cptr] = value;
		cptr++;
	}
	Pcx_Struct->clength = cptr;
} // Pcx::compress

// Pcx::save_image saves compressed bitmap
void Pcx::save_image()
{
	compress();
	fwrite(Pcx_Struct->cimage, Pcx_Struct->clength, 1, fp);
} // Pcx::save_image

// Pcx:save_palette loads color reg values from file into palette array
void Pcx::save_palette(FILE *fp)
{
	int i, j;
	char Code256 = PCX_256PALETTE;

	fwrite(&Code256, 1, 1, fp);
	// Adjust for required bit shifts
	for(i=0;i<256;i++)
		for(j=0;j<3;j++)
			Pcx_Struct->palette[i*3+j] <<= 2;
	// write all 768 bytes of palette
	fwrite(Pcx_Struct->palette, 3*256, 1, fp);
	// Adjust for required bit shifts
	for(i=0;i<256;i++)
		for(j=0;j<3;j++)
			Pcx_Struct->palette[i*3+j] >>= 2;
} // Pcx::save_palette

// Pcx::save saves PCX data from structure PCX into file FILENAME
int Pcx::save(char *filename)
{
	long res;

	fp = fopen(filename, "wb");
	if(fp == NULL)
		return (NO_FILE_OPEN);

	// Write header
	res = fwrite(&(Pcx_Struct->header), sizeof(pcx_header), 1, fp);
	if(res != 1)
	{
		fclose(fp);
		return(NO_FILE_WRITE);
	}

	// Allocate memory for CIMAGE, return error if memory not available
	if((Pcx_Struct->cimage = new BYTE[IMAGE_SIZE]) == NULL)
	{
		fclose(fp);
		return (NO_MEM);
	}
	save_image();
	delete Pcx_Struct->cimage;

	// save palette
	save_palette(fp);

	fclose(fp);
	return GOOD;
} // Pcx::save

int Pcx::New()
{
	if(image_loaded)
		delete Pcx_Struct->image;
	delete Pcx_Struct;
	Pcx_Struct = new struct pcx_struct;
	if((Pcx_Struct->image = new BYTE[IMAGE_SIZE]) == NULL)
		return (NO_MEM);
	image_loaded = 1;
	memset(Pcx_Struct->image, 0, IMAGE_SIZE);    // set it all to zero
	memset(&(Pcx_Struct->header), 0, sizeof(struct pcx_header));
	Pcx_Struct->header.manufacturer = PCX_MANUFACTURER;
	Pcx_Struct->header.version = PCX_VERSION;
	Pcx_Struct->header.encoding = PCX_ENCODING;
	Pcx_Struct->header.bits_per_pixel = 8;
	Pcx_Struct->header.xmin = 0;
	Pcx_Struct->header.ymin = 0;
	Pcx_Struct->header.xmax = MAXX;
	Pcx_Struct->header.ymax = MAXY;
	Pcx_Struct->header.hres = MAXX+1;
	Pcx_Struct->header.vres = MAXY+1;
	Pcx_Struct->header.reserved = 0;
	Pcx_Struct->header.color_planes = 1;
	Pcx_Struct->header.bytes_per_line = MAXX+1;
	Pcx_Struct->header.palette_type = 0;
	if((infile = open("default.pal", O_BINARY)) == NULL)
		return (NO_DEFAULT_PAL);
	load_palette(infile);
	close(infile);
	return GOOD;
} // Pcx::New

int Pcx::Show()
{
	if(Visible)
		return 0;
	temp = new BYTE[IMAGE_SIZE];
	if(temp==NULL)
		return NO_MEM;
	memcpy(temp, screen, IMAGE_SIZE);
	memcpy(screen, Pcx_Struct->image, IMAGE_SIZE);
	Visible = 1;
	return GOOD;
} // Pcx::Show

void Pcx::Hide()
{
	if(!Visible)
		return;
	memcpy(screen, temp, IMAGE_SIZE);
	delete temp;
	Visible = 0;
} // Pcx::Hide

Pcx::~Pcx()
{
	if(Visible)
		Hide();
	if(image_loaded)
		delete Pcx_Struct->image;
	delete Pcx_Struct;
} // Pcx destructor
